/*
 *
 * Copyright (C) 2008 BLADOX, s.r.o.  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.    
*/

#include <config.h>
#include <turbo/turbo.h>

#include <stdlib.h>
#include <string.h>

#define VERSION_A	0
#define VERSION_B	9

/* *INDENT-OFF* */

static lc_char PROGMEM lc_CB[]={
        LC_EN("Callback")
        LC_END
};

static lc_char PROGMEM lc_Active[]={
        LC_EN("Active")
        LC_END
};

static lc_char PROGMEM lc_Service_Nr[]={
        LC_EN("Service Nr")
        LC_END
};

static lc_char PROGMEM lc_About[]={
        LC_EN("About")
        LC_END
};

lc_char PROGMEM lc_Yes[]={
        LC_EN("  Yes")
        LC_CZ("  Ano")
        LC_FR("  Oui")
        LC_DE("  Ja")
        LC_END
};

lc_char PROGMEM lc_No[]={
        LC_EN("  No")
        LC_CZ("  Ne")
        LC_FR("  Non")
        LC_DE("  Nein")
        LC_END
};


/* *INDENT-ON* */

u8 *gen_buf;
u8 *get_resp_call;
u8 get_resp_call_len;
u8 is_call;
u8 send_connected;

#define MAX_NR_LEN      30

u8 PROGMEM t_bladox_site[] = "www.bladox.com\n";

typedef struct _Pers_mem
{
  u8 active;
  u8 service_nr[MAX_NR_LEN];
}
Pers_mem;

Pers_mem *pers_mem;

u8 char2ebcd (u8 x)
{
  if (x >= '0' && x <= '9')
    x = x - '0';
  else if (x == '*')
    x = 0x0A;
  else if (x == 0x3A)
    x = 0x0A;
  else if (x == '#')
    x = 0x0B;
  else if (x == 0x3B)
    x = 0x0B;
  else if (x == 'C')
    x = 0x0C;
  else if (x == '+')
    x = 0x0C;
  else if (x == 0x3C)
    x = 0x0C;
  else if (x == '?')
    x = 0x0D;
  else if (x == 'E')
    x = 0x0E;
  else
    x = 0x0F;
  return x;
}

#define CALL_STAT_ALLOW         0x00
#define CALL_STAT_CANCEL        0x01
#define CALL_STAT_MODIFY        0x02

void call_control (u8 * s)
{
  u8 *ms;
  u8 ms_len;
  u8 i;
  u8 j;
  u8 voicemail = 0;
  u8 stat;
  u16 ret;
  u8 em = 0;

  get_resp_call_len = 0;
  stat = CALL_STAT_ALLOW;

  j = get_tag (s, T_ADDRESS);
  if (j != 0 && rb (&pers_mem->active)
      && rb (&pers_mem->service_nr[0]) != 0x00)
  {
    u8 x, y;
    u8 *ms;
    u8 ms_len;

    j++;
    ms = msisdn2str (&s[j], MSISDN_ADN, MEM_R);
    ms_len = strlen (ms);

    stat = CALL_STAT_MODIFY;
    get_resp_call[get_resp_call_len++] = CALL_STAT_MODIFY;
    get_resp_call[get_resp_call_len++] = 0x00;	//len
    get_resp_call[get_resp_call_len++] = T_ADDRESS;
    get_resp_call[get_resp_call_len++] = 0x00;	//len
    get_resp_call[get_resp_call_len++] = 0x81;

    i = 0;

    while (rb (&pers_mem->service_nr[i]) != 0x00)
    {
      gen_buf[i] = rb (&pers_mem->service_nr[i]);
      i++;
    }
    gen_buf[i] = '\0';

    for (j = 0; j < i; j = j + 2)
    {
      x = char2ebcd (gen_buf[j]);
      y = char2ebcd (gen_buf[j + 1]);
      y = y << 4;
      x = x | y;
      get_resp_call[get_resp_call_len++] = x;
    }

    memcpy (gen_buf, ms, ms_len);
    gen_buf[ms_len++] = '#';
    gen_buf[ms_len++] = '\0';
    send_connected = 2;

    get_resp_call[3] = get_resp_call_len - 4;
    get_resp_call[1] = get_resp_call_len - 2;

    free (ms);
  }

  if (stat == CALL_STAT_ALLOW)
  {
    get_resp_call[get_resp_call_len] = CALL_STAT_ALLOW;
    get_resp_call_len++;
    get_resp_call[get_resp_call_len] = 0x00;
    get_resp_call_len++;
  }

  if (stat == CALL_STAT_CANCEL)
  {
    get_resp_call[get_resp_call_len] = CALL_STAT_CANCEL;
    get_resp_call_len++;
    get_resp_call[get_resp_call_len] = 0x00;
    get_resp_call_len++;
  }

  ret = 0x9F00 | get_resp_call_len;
  dbsp ("CALL RET ");
  dbih (ret);
  dbc ('\n');

  get_resp_call[get_resp_call_len] = 0x90;
  get_resp_call_len++;
  get_resp_call[get_resp_call_len] = 0x00;
  get_resp_call_len++;

  retval (ret);
}

u8 ABC_val (u8 * ret, u8 val, u8 * text, u8 * A_text, u8 * B_text,
	    u8 * C_text)
{
  u8 *res;
  u8 j, n;
  u8 t_A[15];
  u8 t_B[15];
  u8 t_C[15];
  u8 *t_list[3];

  t_list[0] = t_A;
  t_list[1] = t_B;
  t_list[2] = t_C;

  memcpy (t_A, A_text, strlen (A_text) + 1);
  memcpy (t_B, B_text, strlen (B_text) + 1);
  n = 2;
  if (C_text)
  {
    memcpy (t_C, C_text, strlen (C_text) + 1);
    n = 3;
  }
  if (val == 0)
  {
    t_A[0] = '*';
  }
  else if (val == 1)
  {
    t_B[0] = '*';
  }
  else
  {
    t_C[0] = '*';
  }

  j = val + 1;

  res = select_item (n, t_list, text, NULL, j, Q_SELECT_ITEM_CHOICE);
  j = get_tag (res, T_RESULT);
  if (j != 0)
  {
    j++;			// tag
    j++;			//len
    if (res[j] == 0x10)
      return APP_END;		// end
    if (res[j] == 0x11 || res[j] == 0x12)
      return APP_BACK;		// back
  }
  j = get_tag (res, T_ITEM_ID);
  if (j != 0)
  {
    j++;			//tag
    j++;			//len
    if (res[j] == 1)
      *ret = 0;
    else if (res[j] == 2)
      *ret = 1;
    else
      *ret = 2;
  }
  return APP_OK;
}

u8 cb_service_nr (SCtx * ctx, u8 action)
{
  if (action == APP_ENTER)
  {
    u8 *res;
    u8 i;

    res =
      get_input (locale (lc_Service_Nr), 2, MAX_NR_LEN - 1, NULL,
		 Q_GET_INPUT_DIGITS);

    if (res == ENULL)
      return APP_END;		// APP_END;

    if (res == NULL)
      return APP_BACK;		// APP_BACK;

    res[res[0] + 1] = '\0';

    if (res[2] == '+')
    {
      i = 3;
      wb (&pers_mem->service_nr[0], '0');
      wb (&pers_mem->service_nr[1], '0');
      memcpy (&pers_mem->service_nr[2], &res[i], strlen (&res[i]) + 1);
    }
    else
    {
      i = 2;			// 00
      memcpy (&pers_mem->service_nr[0], &res[i], strlen (&res[i]) + 1);
    }

    return APP_BACK;
  }
  return APP_OK;
}

u8 cb_active (SCtx * ctx, u8 action)
{
  if (action == APP_ENTER)
  {
    u8 res, ret, val;

    val = rb (&pers_mem->active);

    res = ABC_val (&ret, val, NULL, locale (lc_No), locale (lc_Yes), NULL);

    if (res != APP_OK)
      return res;

    if (ret != val)
    {
      if (ret)
	wb (&pers_mem->active, 1);
      else
	wb (&pers_mem->active, 0);
    }

    return APP_BACK;
  }
  return APP_OK;
}

u8 cb_about (SCtx * ctx, u8 action)
{
  if (action == APP_ENTER)
  {
    u8 *buf = buf_B ();
    u8 *r = buf;
    u8 i;

    r = sprints (r, locale (lc_Service_Nr));
    r = sprintc (r, ' ');
    i = 0;
    while (rb (&pers_mem->service_nr[i]) != 0x00)
    {
      r = sprintc (r, rb (&pers_mem->service_nr[i]));
      i++;
    }
    r = sprintc (r, '\n');
    r = sprintc (r, '\n');

    r = sprints (r, locale (lc_CB));
    r = sprintc (r, '-');
    r = sprinti (r, VERSION_A);
    r = sprintc (r, '.');
    r = sprinti (r, VERSION_B);
    r = sprintc (r, '\n');

    r = sprints (r, t_bladox_site);

    r = sprintc (r, '\0');

    i = display_text (buf, NULL);
    if (i != APP_END)
      return APP_BACK;
    return i;
  }
  return APP_OK;
}

void send_cb_dtmf (u8 * s)
{
// send_dtmf (nr_cb, locale (lc_Please_Wait));

  u8 *buf = buf_A ();
  u8 len;
  u8 i, l, j, k;
  u8 *res = NULL;
  u8 *ms = gen_buf;

#define MAX_DTMF        8

  l = strlen (ms);
  k = 0;

  while (is_call && k < l)
  {
  X1:

    len = 0;
    buf[len++] = T_DTMF_STRING | CR_FLAG;

    if (k + MAX_DTMF <= l)
      j = MAX_DTMF;
    else
    {
      if ((l - k) % 2)
	j = l - k + 1;
      else
	j = l - k;
    }

    buf[len++] = j / 2;

    for (i = 0; i < j; i += 2)
      buf[len++] = (char2ebcd (ms[k + i + 1]) << 4) | char2ebcd (ms[k + i]);

    res = stk_cmd (STK_CMD_SEND_DTMF, 0x00, DEV_ID_NETWORK, len);

    j = get_tag (res, T_RESULT);
    j++;			// tag
    j++;			// len
    if (res[j] != 0 && is_call == 1)
    {
      dbsp ("DTMF RESEND\n");
      goto X1;
    }

    if (res[j] == 0)
    {
      k += MAX_DTMF;
    }
  }
}

SNodeP cb_n = { lc_CB, NULL };
SNodeP cb_active_n = { lc_Active, cb_active };
SNodeP cb_service_nr_n = { lc_Service_Nr, cb_service_nr };
SNodeP cb_about_n = { lc_About, cb_about };

/* *INDENT-OFF* */

SEdgeP cb_edges_p[] = {
  {&cb_n, &cb_active_n},
  {&cb_n, &cb_service_nr_n},
  {&cb_n, &cb_about_n},
  NULL
};

/* *INDENT-ON* */

void action_menu (Menu_selection_data * x)
{
  SCtx *c;

  c = spider_init ();
  c->eP = &cb_edges_p;
  c->n = &cb_n;
  spider (c);
}

void turbo_handler (u8 action, void *data)
{
  switch (action)
  {
    case ACTION_APP_REGISTER:
      {
	Pers_mem *p = emalloc (sizeof (Pers_mem));
	u8 i;

	pers_mem = p;
	wb (&p->active, 0);
	for (i = 0; i < MAX_NR_LEN; i++)
	  wb (&p->service_nr[i], 0);

	reg_app_data (p);

      }
      break;

    case ACTION_APP_UNREGISTER:
      {
	Pers_mem *p = app_data ();

	efree (p);
      }
      break;

    case ACTION_APP_INIT:
      pers_mem = app_data ();

      gen_buf = malloc (50);
      get_resp_call = malloc (50);

      reg_action (ACTION_CALL_CONTROL);
      reg_action (ACTION_EVENT_MT_CALL);
      reg_action (ACTION_EVENT_CALL_CONNECTED);
      reg_action (ACTION_EVENT_CALL_DISCONNECTED);

      is_call = 0;
      send_connected = 0;

      break;

    case ACTION_INSERT_MENU:
      insert_menu (locale (lc_CB));
      break;

    case ACTION_MENU_SELECTION:
      stk_thread (action_menu, NULL);
      break;

    case ACTION_EVENT_CALL_CONNECTED:
      dbsp ("ACTION_EVENT_CALL_CONNECTED\n");
      is_call = 1;
      if (send_connected)
      {
	stk_thread (send_cb_dtmf, NULL);
	send_connected = 0;
      }
      break;

    case ACTION_EVENT_CALL_DISCONNECTED:
      dbsp ("ACTION_EVENT_CALL_DISCONNECTED\n");
      is_call = 0;
      if (send_connected)
	send_connected--;
      break;

    case ACTION_EVENT_MT_CALL:
      dbsp ("ACTION_EVENT_MT_CALL\n");
      is_call = 1;
      break;

    case ACTION_CALL_CONTROL:
      call_control (data);
      break;

    case ACTION_FILE_APDU:
      if (((File_apdu_data *) data)->prev_ins == ME_CMD_ENVELOPE
	  && ((File_apdu_data *) data)->ins == ME_CMD_GET_RESPONSE)
      {
	u8 *r = buf_B ();
	u8 i;

	dbsp ("CALL FILE APDU\n");
	for (i = 0; i < get_resp_call_len; i++)
	  r[i] = get_resp_call[i];
      }
      break;

    default:
      break;
  }
}
